@@ -15,7 +15,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: egg-editable-toolbar.c,v 1.44 2004/02/09 14:15:13 chpe Exp $
+ *  $Id: egg-editable-toolbar.c,v 1.48 2004/03/14 01:01:03 marco Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -450,6 +450,8 @@ drag_data_received_cb (GtkWidget          *widget,
       etoolbar->priv->pending = FALSE;
       etoolbar->priv->dragged_item =
         create_item_from_action (etoolbar, id, data_is_separator (id), NULL);
+      g_object_ref (etoolbar->priv->dragged_item);
+      gtk_object_sink (GTK_OBJECT (etoolbar->priv->dragged_item));
     }
   else
     {
@@ -519,7 +521,7 @@ popup_toolbar_context_menu_cb (GtkWidget          *toolbar,
 
       position = get_toolbar_position (t, toolbar);
       flags = egg_toolbars_model_get_flags (t->priv->model, position);
-      if (flags && EGG_TB_MODEL_NOT_REMOVABLE)
+      if (flags & EGG_TB_MODEL_NOT_REMOVABLE)
         {
           gtk_widget_set_sensitive (GTK_WIDGET (item), FALSE);
         }
@@ -595,23 +597,16 @@ toolbar_drag_motion_cb (GtkWidget          *widget,
 
       etoolbar->priv->target_toolbar = toolbar;
 
-      /* The handler will make sure the item is created */
       gtk_drag_get_data (widget, context, target, time);
-
-      g_assert (!etoolbar->priv->pending);
-
-      if (etoolbar->priv->dragged_item == NULL) {
-	return TRUE;
-      }
-
-      g_object_ref (etoolbar->priv->dragged_item);
-      gtk_object_sink (GTK_OBJECT (etoolbar->priv->dragged_item));
     }
 
-  item = GTK_TOOL_ITEM (etoolbar->priv->dragged_item);
+  if (etoolbar->priv->dragged_item != NULL)
+    {
+      item = GTK_TOOL_ITEM (etoolbar->priv->dragged_item);
 
-  index = gtk_toolbar_get_drop_index (toolbar, x, y);
-  gtk_toolbar_set_drop_highlight_item (toolbar, item, index);
+      index = gtk_toolbar_get_drop_index (toolbar, x, y);
+      gtk_toolbar_set_drop_highlight_item (toolbar, item, index);
+    }
 
   gdk_drag_status (context, context->suggested_action, time);
 
@@ -784,16 +779,20 @@ item_added_cb (EggToolbarsModel   *model,
 	       int                 position,
 	       EggEditableToolbar *t)
 {
+  GtkWidget *dock;
   GtkWidget *toolbar;
   GtkWidget *item;
   GtkAction *action;
 
   toolbar = get_toolbar_nth (t, toolbar_position);
-  gtk_widget_set_size_request (toolbar, -1, -1);
   item = create_item (t, model, toolbar_position, position, &action);
   gtk_toolbar_insert (GTK_TOOLBAR (toolbar),
 		      GTK_TOOL_ITEM (item), position);
 
+  dock = get_dock_nth (t, toolbar_position);
+  gtk_widget_set_size_request (dock, -1, -1);
+  gtk_widget_queue_resize_no_redraw (dock);
+
   /* FIXME Hack to make tooltip work from gtk */
   if (action)
     {
@@ -818,7 +817,6 @@ item_removed_cb (EggToolbarsModel   *model,
 
   if (egg_toolbars_model_n_items (model, toolbar_position) == 0)
     {
-      gtk_widget_set_size_request (toolbar, -1, MIN_TOOLBAR_HEIGHT);
       egg_toolbars_model_remove_toolbar (model, toolbar_position);
     }
 }
@@ -897,7 +895,7 @@ egg_editable_toolbar_construct (EggEditableToolbar *t)
 
       if (n_items == 0)
         {
-            gtk_widget_set_size_request (toolbar, -1, MIN_TOOLBAR_HEIGHT);
+            gtk_widget_set_size_request (dock, -1, MIN_TOOLBAR_HEIGHT);
         }
     }
 
@@ -1050,6 +1048,7 @@ egg_editable_toolbar_init (EggEditableToolbar *t)
 
   t->priv->merge = NULL;
   t->priv->edit_mode = FALSE;
+  t->priv->dragged_item = NULL;
 }
 
 static void
@@ -1085,6 +1084,12 @@ egg_editable_toolbar_new (GtkUIManager     *merge,
 				   NULL));
 }
 
+gboolean
+egg_editable_toolbar_get_edit_mode (EggEditableToolbar *etoolbar)
+{
+	return etoolbar->priv->edit_mode;
+}
+
 void
 egg_editable_toolbar_set_edit_mode (EggEditableToolbar *etoolbar,
 				    gboolean            mode)
