@@ -15,7 +15,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-stock-icons.h,v 1.9 2003/11/22 14:40:37 marco Exp $
+ *  $Id: ephy-stock-icons.h,v 1.11 2004/02/01 22:45:22 marco Exp $
  */
 
 #ifndef EPHY_STOCK_ICONS_H
@@ -33,8 +33,9 @@ G_BEGIN_DECLS
 #define STOCK_NEW_TAB		   "stock_new-tab"
 #define STOCK_FULLSCREEN	   "stock_fullscreen"
 #define STOCK_VIEW_SOURCE	   "stock_view-html-source"
-#define STOCK_SEND_MAIL		   "stock_mail_send"
+#define STOCK_SEND_MAIL		   "stock_mail-send"
 #define STOCK_ADD_BOOKMARK	   "stock_add-bookmark"
+#define STOCK_PRINT_SETUP	   "stock_print-setup"
 
 void ephy_stock_icons_init (void);
 
