@@ -1,6 +1,7 @@
 /*
  *  Copyright (C) 2000-2003 Marco Pesenti Gritti
- *  Copyright (C) 2003 Christian Persch
+ *  Copyright (C) 2003, 2004 Christian Persch
+ *  Copyright (C) 2004 Crispin Flowerday
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -16,7 +17,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-tab.c,v 1.76 2003/12/19 14:35:55 chpe Exp $
+ *  $Id: ephy-tab.c,v 1.95 2004/03/04 23:09:02 chpe Exp $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -24,7 +25,6 @@
 #endif
 
 #include "ephy-tab.h"
-#include "ephy-shell.h"
 #include "eel-gconf-extensions.h"
 #include "ephy-prefs.h"
 #include "ephy-embed-factory.h"
@@ -36,6 +36,9 @@
 #include "ephy-zoom.h"
 #include "ephy-favicon-cache.h"
 #include "ephy-embed-persist.h"
+#include "ephy-history.h"
+#include "ephy-embed-shell.h"
+#include "ephy-shell.h"
 
 #include <glib/gi18n.h>
 #include <libgnomevfs/gnome-vfs-uri.h>
@@ -52,13 +55,13 @@
 #include <gtk/gtkmenu.h>
 #include <gtk/gtkuimanager.h>
 #include <gtk/gtkradioaction.h>
+#include <gtk/gtkclipboard.h>
 #include <string.h>
 
 #define EPHY_TAB_GET_PRIVATE(object)(G_TYPE_INSTANCE_GET_PRIVATE ((object), EPHY_TYPE_TAB, EphyTabPrivate))
 
 struct EphyTabPrivate
 {
-	EphyEmbed *embed;
 	EphyWindow *window;
 	char *status_message;
 	char *link_message;
@@ -113,10 +116,12 @@ static void	ephy_tab_set_link_message	(EphyTab *tab,
 						 char *message);
 static void	ephy_tab_set_load_percent	(EphyTab *tab,
 						 int percent);
-static void	ephy_tab_update_navigation_flags(EphyTab *tab);
+static void	ephy_tab_update_navigation_flags(EphyTab *tab,
+						 EphyEmbed *embed);
 static void	ephy_tab_set_security_level	(EphyTab *tab,
 						 EmbedSecurityLevel level);
 static void	ephy_tab_set_title		(EphyTab *tab,
+						 EphyEmbed *embed,
 						 const char *new_title);
 static void	ephy_tab_set_zoom		(EphyTab *tab,
 						 float zoom);
@@ -143,7 +148,7 @@ ephy_tab_get_type (void)
                         (GInstanceInitFunc) ephy_tab_init
                 };
 
-                ephy_tab_type = g_type_register_static (G_TYPE_OBJECT,
+                ephy_tab_type = g_type_register_static (GTK_TYPE_BIN,
 							"EphyTab",
 							&our_info, 0);
         }
@@ -229,6 +234,40 @@ ephy_tab_get_property (GObject *object,
 	}
 }
 
+static void
+ephy_tab_size_allocate (GtkWidget *widget,
+			GtkAllocation *allocation)
+{
+	GtkWidget *child;
+
+	widget->allocation = *allocation;
+
+	child = GTK_BIN (widget)->child;
+
+	if (child && GTK_WIDGET_VISIBLE (child))
+	{
+		gtk_widget_size_allocate (child, allocation);
+	}
+}
+
+static void
+ephy_tab_parent_set (GtkWidget *widget,
+		     GtkWidget *previous_parent)
+{
+	if (widget->parent)
+	{
+		GtkWidget *toplevel;
+		toplevel = gtk_widget_get_toplevel (widget);
+
+		ephy_tab_set_window (EPHY_TAB (widget), EPHY_WINDOW (toplevel));
+	}
+
+	if (GTK_WIDGET_CLASS (parent_class)->parent_set)
+	{
+		GTK_WIDGET_CLASS (parent_class)->parent_set (widget, previous_parent);
+	}
+}
+
 static void
 ephy_tab_action_activate_cb (GtkAction *action, EphyTab *tab)
 {
@@ -245,6 +284,7 @@ static void
 ephy_tab_class_init (EphyTabClass *class)
 {
         GObjectClass *object_class = G_OBJECT_CLASS (class);
+	GtkWidgetClass *widget_class = GTK_WIDGET_CLASS(class);
 
         parent_class = g_type_class_peek_parent (class);
 
@@ -252,6 +292,9 @@ ephy_tab_class_init (EphyTabClass *class)
 	object_class->get_property = ephy_tab_get_property;
 	object_class->set_property = ephy_tab_set_property;
 
+	widget_class->size_allocate = ephy_tab_size_allocate;
+	widget_class->parent_set = ephy_tab_parent_set;
+	
 	g_object_class_install_property (object_class,
 					 PROP_ADDRESS,
 					 g_param_spec_string ("address",
@@ -269,7 +312,7 @@ ephy_tab_class_init (EphyTabClass *class)
 							      G_PARAM_READABLE));
 
 	g_object_class_install_property (object_class,
-					 PROP_LOAD_STATUS,
+					 PROP_LOAD_PROGRESS,
 					 g_param_spec_int ("load-progress",
 							   "Load progress",
 							   "The tab's load progress in percent",
@@ -353,32 +396,14 @@ ephy_tab_class_init (EphyTabClass *class)
 	g_type_class_add_private (object_class, sizeof(EphyTabPrivate));
 }
 
-static void
-ephy_tab_parent_set_cb (GtkWidget *widget, GtkWidget *previous_parent,
-			EphyTab *tab)
-{
-	GtkWidget *toplevel;
-
-	if (widget->parent == NULL) return;
-
-	toplevel = gtk_widget_get_toplevel (widget);
-	ephy_tab_set_window (tab, EPHY_WINDOW (toplevel));
-}
-static void
-ephy_tab_embed_destroy_cb (GtkWidget *widget, EphyTab *tab)
-{
-	LOG ("GtkMozEmbed destroy signal on EphyTab")
-	g_object_unref (tab);
-}
-
 static void
 ephy_tab_finalize (GObject *object)
 {
         EphyTab *tab = EPHY_TAB (object);
 
-	g_idle_remove_by_data (tab->priv->embed);
+	g_idle_remove_by_data (tab);
 
-	if (tab->priv->action)
+	if (tab->priv->action != NULL)
 	{
 		g_object_unref (tab->priv->action);
 	}
@@ -397,22 +422,17 @@ ephy_tab_finalize (GObject *object)
 static gboolean
 address_has_web_scheme (const char *address)
 {
-	GnomeVFSURI *uri;
-	const char *scheme;
-	gboolean has_web_scheme = FALSE;
+	gboolean has_web_scheme;
 
-	uri = gnome_vfs_uri_new (address);
-	if (uri != NULL)
-	{
-		scheme = gnome_vfs_uri_get_scheme (uri);
+	if (address == NULL) return FALSE;
 
-		has_web_scheme = (strcmp (scheme, "http") == 0 ||
-				  strcmp (scheme, "https") == 0 ||
-				  strcmp (scheme, "ftp") == 0 ||
-				  strcmp (scheme, "file") == 0);
-
-		gnome_vfs_uri_unref (uri);
-	}
+	has_web_scheme = (g_str_has_prefix (address, "http:") ||
+			  g_str_has_prefix (address, "https:") ||
+			  g_str_has_prefix (address, "ftp:") ||
+			  g_str_has_prefix (address, "file:") ||
+			  g_str_has_prefix (address, "data:") ||
+			  g_str_has_prefix (address, "about:") ||
+			  g_str_has_prefix (address, "gopher:"));
 
 	return has_web_scheme;
 }
@@ -422,11 +442,7 @@ address_has_web_scheme (const char *address)
 EphyTab *
 ephy_tab_new (void)
 {
-	EphyTab *tab;
-
-	tab = EPHY_TAB (g_object_new (EPHY_TYPE_TAB, NULL));
-
-	return tab;
+	return EPHY_TAB (g_object_new (EPHY_TYPE_TAB, NULL));
 }
 
 static void
@@ -471,7 +487,20 @@ ephy_tab_get_embed (EphyTab *tab)
 {
 	g_return_val_if_fail (EPHY_IS_TAB (tab), NULL);
 
-	return tab->priv->embed;
+	return EPHY_EMBED (gtk_bin_get_child (GTK_BIN (tab)));
+}
+
+EphyTab *
+ephy_tab_for_embed (EphyEmbed *embed)
+{
+	GtkWidget *parent;
+
+	g_return_val_if_fail (EPHY_IS_EMBED (embed), NULL);
+
+	parent = GTK_WIDGET (embed)->parent;
+	g_return_val_if_fail (parent != NULL, NULL);
+
+	return EPHY_TAB (parent);
 }
 
 void
@@ -555,8 +584,8 @@ ephy_tab_set_icon_address (EphyTab *tab, const char *address)
 	if (tab->priv->icon_address)
 	{
 		eb = ephy_shell_get_bookmarks (ephy_shell);
-		history = ephy_embed_shell_get_global_history
-			(EPHY_EMBED_SHELL (ephy_shell));
+		history = EPHY_HISTORY
+			(ephy_embed_shell_get_global_history (embed_shell));
 		ephy_bookmarks_set_icon (eb, tab->priv->address,
 				         tab->priv->icon_address);
 		ephy_history_set_icon (history, tab->priv->address,
@@ -594,16 +623,29 @@ ephy_tab_link_message_cb (EphyEmbed *embed,
 static void
 ephy_tab_address_cb (EphyEmbed *embed, const char *address, EphyTab *tab)
 {
+	const char *uv_address;
+
 	LOG ("ephy_tab_address_cb tab %p address %s", tab, address)
 
+	/* Do not expose about:blank to the user, an empty address
+	   bar will do better */
+	if (address && strcmp (address, "about:blank") == 0)
+	{
+		uv_address = "";
+	}
+	else
+	{
+		uv_address = address;
+	}
+
 	if (tab->priv->address_expire == TAB_ADDRESS_EXPIRE_NOW)
 	{
-		ephy_tab_set_location (tab, address, TAB_ADDRESS_EXPIRE_NOW);
+		ephy_tab_set_location (tab, uv_address, TAB_ADDRESS_EXPIRE_NOW);
 	}
 
 	ephy_tab_set_link_message (tab, NULL);
 	ephy_tab_set_icon_address (tab, NULL);
-	ephy_tab_update_navigation_flags (tab);
+	ephy_tab_update_navigation_flags (tab, embed);
 
 	/* restore zoom level */
 	if (address_has_web_scheme (address))
@@ -613,8 +655,8 @@ ephy_tab_address_cb (EphyEmbed *embed, const char *address, EphyTab *tab)
 		GValue value = { 0, };
 		float zoom = 1.0, current_zoom;
 
-		history = ephy_embed_shell_get_global_history
-				(EPHY_EMBED_SHELL (ephy_shell));
+		history = EPHY_HISTORY
+			(ephy_embed_shell_get_global_history (embed_shell));
 		host = ephy_history_get_host (history, address);
 
 		if (host != NULL && ephy_node_get_property
@@ -654,8 +696,8 @@ ephy_tab_zoom_changed_cb (EphyEmbed *embed, float zoom, EphyTab *tab)
 		EphyHistory *history;
 		EphyNode *host;
 		GValue value = { 0, };
-		history = ephy_embed_shell_get_global_history
-				(EPHY_EMBED_SHELL (ephy_shell));
+		history = EPHY_HISTORY
+			(ephy_embed_shell_get_global_history (embed_shell));
 		host = ephy_history_get_host (history, address);
 
 		if (host != NULL)
@@ -682,7 +724,7 @@ ephy_tab_title_cb (EphyEmbed *embed, EphyTab *tab)
 
 	title = ephy_embed_get_title (embed);
 
-	ephy_tab_set_title (tab, title);
+	ephy_tab_set_title (tab, embed, title);
 
 	g_free (title);
 }
@@ -813,9 +855,9 @@ build_progress_from_requests (EphyTab *tab, EmbedState state)
 }
 
 static void
-ensure_page_info (EphyTab *tab, const char *address)
+ensure_page_info (EphyTab *tab, EphyEmbed *embed, const char *address)
 {
-	if (tab->priv->address == NULL &&
+	if ((tab->priv->address == NULL || *tab->priv->address == '\0') &&
 	    tab->priv->address_expire == TAB_ADDRESS_EXPIRE_NOW)
         {
 		ephy_tab_set_location (tab, address, TAB_ADDRESS_EXPIRE_NOW);
@@ -823,7 +865,7 @@ ensure_page_info (EphyTab *tab, const char *address)
 
 	if (tab->priv->title == NULL)
 	{
-		ephy_tab_set_title (tab, NULL);
+		ephy_tab_set_title (tab, embed, NULL);
 	}
 }
 
@@ -839,17 +881,17 @@ ephy_tab_net_state_cb (EphyEmbed *embed, const char *uri,
 		{
 			tab->priv->total_requests = 0;
 			tab->priv->cur_requests = 0;
-			ensure_page_info (tab, uri);
+			ensure_page_info (tab, embed, uri);
 
 			ephy_tab_set_load_percent (tab, 0);
 			ephy_tab_set_load_status (tab, TRUE);
-			ephy_tab_update_navigation_flags (tab);
+			ephy_tab_update_navigation_flags (tab, embed);
 		}
 		else if (state & EMBED_STATE_STOP)
 		{
 			ephy_tab_set_load_percent (tab, 0);
 			ephy_tab_set_load_status (tab, FALSE);
-			ephy_tab_update_navigation_flags (tab);
+			ephy_tab_update_navigation_flags (tab, embed);
 			tab->priv->address_expire = TAB_ADDRESS_EXPIRE_NOW;
 		}
 	}
@@ -868,30 +910,32 @@ ephy_tab_new_window_cb (EphyEmbed *embed, EphyEmbed **new_embed,
 	ephy_window_request_chrome (window, chromemask);
 
 	new_tab = ephy_tab_new ();
+	gtk_widget_show (GTK_WIDGET (new_tab));
+
         ephy_window_add_tab (window, new_tab, EPHY_NOTEBOOK_INSERT_GROUPED, FALSE);
 
 	*new_embed = ephy_tab_get_embed (new_tab);
 }
 
 static gboolean
-let_me_resize_hack (gpointer data)
+let_me_resize_hack (GtkWidget *tab)
 {
-	gtk_widget_set_size_request (GTK_WIDGET(data),
-				     -1, -1);
+	gtk_widget_set_size_request (tab, -1, -1);
+
 	return FALSE;
 }
 
 static void
 ephy_tab_visibility_cb (EphyEmbed *embed, gboolean visibility,
-			  EphyTab *tab)
+			EphyTab *tab)
 {
 	if (visibility)
 	{
-		gtk_widget_show (GTK_WIDGET(embed));
+		gtk_widget_show (GTK_WIDGET (tab));
 	}
 	else
 	{
-		gtk_widget_hide (GTK_WIDGET(embed));
+		gtk_widget_hide (GTK_WIDGET (tab));
 	}
 
 	ephy_tab_set_visibility (tab, visibility);
@@ -900,10 +944,15 @@ ephy_tab_visibility_cb (EphyEmbed *embed, gboolean visibility,
 static void
 ephy_tab_destroy_brsr_cb (EphyEmbed *embed, EphyTab *tab)
 {
+	GtkWidget *notebook;
+
 	g_return_if_fail (EPHY_IS_TAB (tab));
 	g_return_if_fail (tab->priv->window != NULL);
 
-	ephy_window_remove_tab (tab->priv->window, tab);
+	/* Do not use ephy_window_remove_tab because it will
+	   check for unsubmitted forms */
+	notebook = ephy_window_get_notebook (tab->priv->window);
+	ephy_notebook_remove_tab (EPHY_NOTEBOOK (notebook), tab);
 }
 
 static void
@@ -912,14 +961,12 @@ ephy_tab_size_to_cb (EphyEmbed *embed, gint width, gint height,
 {
 	GtkWidget *notebook;
 	EphyWindow *window;
-	GtkWidget *widget;
 
 	tab->priv->width = width;
 	tab->priv->height = height;
 
 	window = tab->priv->window;
 	notebook = ephy_window_get_notebook (window);
-	widget = GTK_WIDGET (embed);
 
 	/* Do not resize window with multiple tabs.
 	 * Do not resize window already showed because
@@ -929,18 +976,19 @@ ephy_tab_size_to_cb (EphyEmbed *embed, gint width, gint height,
 	    !tab->priv->visibility)
 	{
 		gtk_widget_set_size_request
-			(widget, width, height);
+			(GTK_WIDGET (tab), width, height);
 
 		/* HACK reset widget requisition after the container
 		 * has been resized. It appears to be the only way
 		 * to have the window sized according to embed
 		 * size correctly.
 		 */
-		g_idle_add (let_me_resize_hack, embed);
+
+		g_idle_add ((GSourceFunc) let_me_resize_hack, tab);
 	}
 }
 
-static void
+static gboolean
 open_link_in_new_tab (EphyTab *tab,
 		      const char *link_address)
 {
@@ -948,7 +996,7 @@ open_link_in_new_tab (EphyTab *tab,
 	gboolean new_tab;
 
 	window = ephy_tab_get_window (tab);
-	g_return_if_fail (window != NULL);
+	g_return_val_if_fail (window != NULL, FALSE);
 
 	new_tab = address_has_web_scheme (link_address);
 
@@ -959,10 +1007,8 @@ open_link_in_new_tab (EphyTab *tab,
 				    EPHY_NEW_TAB_OPEN_PAGE |
 				    EPHY_NEW_TAB_IN_EXISTING_WINDOW);
 	}
-	else
-	{
-		ephy_window_load_url (window, link_address);
-	}
+
+	return new_tab;
 }
 
 static void
@@ -991,6 +1037,26 @@ save_property_url (EphyEmbed *embed,
 	g_object_unref (G_OBJECT(persist));
 }
 
+static void
+clipboard_text_received_cb (GtkClipboard *clipboard,
+			    const char *text,
+			    gpointer *weak_ptr)
+{
+	if (*weak_ptr != NULL && text != NULL)
+	{
+		EphyEmbed *embed = (EphyEmbed *) *weak_ptr;
+
+		ephy_embed_load_url (embed, text);
+	}
+
+	if (*weak_ptr != NULL)
+	{
+		g_object_remove_weak_pointer (G_OBJECT (*weak_ptr), weak_ptr);
+	}
+
+	g_free (weak_ptr);
+}
+
 static gboolean
 ephy_tab_dom_mouse_click_cb (EphyEmbed *embed,
 			     EphyEmbedEvent *event,
@@ -1023,8 +1089,9 @@ ephy_tab_dom_mouse_click_cb (EphyEmbed *embed,
 	is_left_click = (type == EPHY_EMBED_EVENT_MOUSE_BUTTON1);
 	is_middle_click = (type == EPHY_EMBED_EVENT_MOUSE_BUTTON2);
 
-	middle_click_opens = eel_gconf_get_boolean
-				(CONF_INTERFACE_MIDDLE_CLICK_OPEN_URL);
+	middle_click_opens =
+		eel_gconf_get_boolean (CONF_INTERFACE_MIDDLE_CLICK_OPEN_URL) &&
+		!eel_gconf_get_boolean (CONF_LOCKDOWN_DISABLE_ARBITRARY_URL);
 
 	is_link = (context & EMBED_CONTEXT_LINK) != 0;
 	is_image = (context & EMBED_CONTEXT_IMAGE) != 0;
@@ -1041,12 +1108,12 @@ ephy_tab_dom_mouse_click_cb (EphyEmbed *embed,
 
 		ephy_embed_event_get_property (event, "link", &value);
 		link_address = g_value_get_string (value);
-		open_link_in_new_tab (tab, link_address);
+		handled = open_link_in_new_tab (tab, link_address);
 	}
 	/* shift+click saves the link target */
 	else if (is_link && is_left_click && with_shift)
 	{
-		save_property_url (embed, event, "link", CONF_STATE_DOWNLOAD_DIR);
+		save_property_url (embed, event, "link", CONF_STATE_SAVE_DIR);
 	}
 	/* shift+click saves the non-link image
 	 * Note: pressing enter to submit a form synthesizes a mouse click event
@@ -1058,10 +1125,22 @@ ephy_tab_dom_mouse_click_cb (EphyEmbed *embed,
 	/* middle click opens the selection url */
 	else if (is_middle_clickable && is_middle_click && middle_click_opens)
 	{
-		gtk_selection_convert (GTK_WIDGET (window),
-				       GDK_SELECTION_PRIMARY,
-				       GDK_SELECTION_TYPE_STRING,
-				       GDK_CURRENT_TIME);
+		/* See bug #133633 for why we do it this way */
+
+		/* We need to make sure we know if the embed is destroyed between
+		 * requesting the clipboard contents, and receiving them.
+		 */
+		gpointer *weak_ptr;
+
+		weak_ptr = g_new (gpointer, 1);
+		*weak_ptr = embed;
+		g_object_add_weak_pointer (G_OBJECT (embed), weak_ptr);
+
+		gtk_clipboard_request_text
+			(gtk_widget_get_clipboard (GTK_WIDGET (embed),
+						   GDK_SELECTION_PRIMARY),
+			 (GtkClipboardTextReceivedFunc) clipboard_text_received_cb,
+			 weak_ptr);
 	}
 	/* we didn't handle the event */
 	else
@@ -1082,7 +1161,7 @@ ephy_tab_security_change_cb (EphyEmbed *embed, EmbedSecurityLevel level,
 static void
 ephy_tab_init (EphyTab *tab)
 {
-	GObject *embed, *embed_widget;
+	GObject *embed;
 	EphyFaviconCache *cache;
 	char *id;
 
@@ -1108,12 +1187,11 @@ ephy_tab_init (EphyTab *tab)
 	tab->priv->setting_zoom = FALSE;
 	tab->priv->address_expire = TAB_ADDRESS_EXPIRE_NOW;
 
-	tab->priv->embed = EPHY_EMBED
-		(ephy_embed_factory_new_object ("EphyEmbed"));
-	g_assert (tab->priv->embed != NULL);
+	embed = ephy_embed_factory_new_object ("EphyEmbed");
+	g_assert (embed != NULL);
 
-	embed = G_OBJECT (tab->priv->embed);
-	embed_widget = G_OBJECT (tab->priv->embed);
+	gtk_container_add (GTK_CONTAINER (tab), GTK_WIDGET (embed));
+	gtk_widget_show (GTK_WIDGET (embed));
 
 	id = g_strdup_printf ("Tab%lu", tab_id++);
 
@@ -1127,51 +1205,42 @@ ephy_tab_init (EphyTab *tab)
 	g_signal_connect (tab->priv->action, "activate",
 			  G_CALLBACK (ephy_tab_action_activate_cb), tab);
 
-	/* set a pointer in the embed's widget back to the tab */
-	g_object_set_data (embed_widget, "EphyTab", tab);
-
-	g_signal_connect (embed_widget, "parent_set",
-			  G_CALLBACK (ephy_tab_parent_set_cb),
-			  tab);
-	g_signal_connect (embed_widget, "destroy",
-			  G_CALLBACK (ephy_tab_embed_destroy_cb),
-			  tab);
-	g_signal_connect (embed, "link_message",
-			  G_CALLBACK (ephy_tab_link_message_cb),
-			  tab);
-	g_signal_connect (embed, "ge_location",
-			  G_CALLBACK (ephy_tab_address_cb),
-			  tab);
-	g_signal_connect (embed, "title",
-			  G_CALLBACK (ephy_tab_title_cb),
-			  tab);
-	g_signal_connect (embed, "ge_zoom_change",
-			  G_CALLBACK (ephy_tab_zoom_changed_cb),
-			  tab);
-	g_signal_connect (embed, "ge_net_state",
-			  G_CALLBACK (ephy_tab_net_state_cb),
-			  tab);
-	g_signal_connect (embed, "ge_new_window",
-			  G_CALLBACK (ephy_tab_new_window_cb),
-			  tab);
-	g_signal_connect (embed, "visibility",
-			  G_CALLBACK (ephy_tab_visibility_cb),
-			  tab);
-	g_signal_connect (embed, "destroy_browser",
-			  G_CALLBACK (ephy_tab_destroy_brsr_cb),
-			  tab);
-	g_signal_connect (embed, "size_to",
-			  G_CALLBACK (ephy_tab_size_to_cb),
-			  tab);
-	g_signal_connect (embed, "ge_dom_mouse_click",
-			  G_CALLBACK (ephy_tab_dom_mouse_click_cb),
-			  tab);
-	g_signal_connect (embed, "ge_security_change",
-			  G_CALLBACK (ephy_tab_security_change_cb),
-			  tab);
-	g_signal_connect (embed, "ge_favicon",
-			  G_CALLBACK (ephy_tab_favicon_cb),
-			  tab);
+	g_signal_connect_object (embed, "link_message",
+				 G_CALLBACK (ephy_tab_link_message_cb),
+				 tab, 0);
+	g_signal_connect_object (embed, "ge_location",
+				 G_CALLBACK (ephy_tab_address_cb),
+				 tab, 0);
+	g_signal_connect_object (embed, "title",
+				 G_CALLBACK (ephy_tab_title_cb),
+				 tab, 0);
+	g_signal_connect_object (embed, "ge_zoom_change",
+				 G_CALLBACK (ephy_tab_zoom_changed_cb),
+				 tab, 0);
+	g_signal_connect_object (embed, "ge_net_state",
+				 G_CALLBACK (ephy_tab_net_state_cb),
+				 tab, 0);
+	g_signal_connect_object (embed, "ge_new_window",
+				 G_CALLBACK (ephy_tab_new_window_cb),
+				 tab, 0);
+	g_signal_connect_object (embed, "visibility",
+				 G_CALLBACK (ephy_tab_visibility_cb),
+				 tab, 0);
+	g_signal_connect_object (embed, "destroy_browser",
+				 G_CALLBACK (ephy_tab_destroy_brsr_cb),
+				 tab, 0);
+	g_signal_connect_object (embed, "size_to",
+				 G_CALLBACK (ephy_tab_size_to_cb),
+				 tab, 0);
+	g_signal_connect_object (embed, "ge_dom_mouse_click",
+				 G_CALLBACK (ephy_tab_dom_mouse_click_cb),
+				 tab, 0);
+	g_signal_connect_object (embed, "ge_security_change",
+				 G_CALLBACK (ephy_tab_security_change_cb),
+				 tab, 0);
+	g_signal_connect_object (embed, "ge_favicon",
+				 G_CALLBACK (ephy_tab_favicon_cb),
+				 tab, 0);
 
 	cache = EPHY_FAVICON_CACHE
 		(ephy_embed_shell_get_favicon_cache (EPHY_EMBED_SHELL (ephy_shell)));
@@ -1202,13 +1271,10 @@ ephy_tab_get_load_percent (EphyTab *tab)
 }
 
 static void
-ephy_tab_update_navigation_flags (EphyTab *tab)
+ephy_tab_update_navigation_flags (EphyTab *tab, EphyEmbed *embed)
 {
-	EphyEmbed *embed;
 	TabNavigationFlags flags = 0;
 
-	embed = tab->priv->embed;
-
 	if (ephy_embed_can_go_up (embed))
 	{
 		flags |= TAB_NAV_UP;
@@ -1262,9 +1328,9 @@ ephy_tab_get_status_message (EphyTab *tab)
 #define MAX_LABEL_LENGTH	32
 
 static void
-ephy_tab_set_title (EphyTab *tab, const char *new_title)
+ephy_tab_set_title (EphyTab *tab, EphyEmbed *embed, const char *new_title)
 {
-	char *title_short = NULL;
+	char *title_short, *title_tmp;
 	char *title = NULL;
 
 	g_return_if_fail (EPHY_IS_TAB (tab));
@@ -1276,7 +1342,7 @@ ephy_tab_set_title (EphyTab *tab, const char *new_title)
 		GnomeVFSURI *uri = NULL;
 		char *address;
 
-		address = ephy_embed_get_location (tab->priv->embed, TRUE);
+		address = ephy_embed_get_location (embed, TRUE);
 
 		if (address)
 		{
@@ -1314,16 +1380,18 @@ ephy_tab_set_title (EphyTab *tab, const char *new_title)
 	tab->priv->title = title;
 
 	title_short = ephy_string_shorten (title, MAX_LABEL_LENGTH);
+	title_tmp = ephy_string_double_underscores (title_short);
 
 	/**
 	 * FIXME: instead of shortening the title here, use an egg action
 	 * which creates menu items with ellipsizing labels
 	 */
 	g_object_set (G_OBJECT (tab->priv->action),
-		      "label", title_short,
+		      "label", title_tmp,
 		      NULL);
 
 	g_free (title_short);
+	g_free (title_tmp);
 
 	g_object_notify (G_OBJECT (tab), "title");
 }
